@@ -187,6 +187,10 @@ QVariant PortModel::data(const QModelIndex &index, int role) const
         {
             return portIconFactory[port->linkState()][port->hasExclusiveControl()];
         }
+        else if ((role == Qt::ForegroundRole))
+        {
+            return port->isDirty() ? QBrush(Qt::red) : QVariant();
+        }
         else
         {
             DBG0("Exit PortModel data 6\n");
