@@ -134,28 +134,28 @@ QVariant PortStatsModel::data(const QModelIndex &index, int role) const
 
             // Statistics
             case e_STAT_FRAMES_RCVD:
-                return quint64(stats.rx_pkts());
+                return QString("%L1").arg(quint64(stats.rx_pkts()));
 
             case e_STAT_FRAMES_SENT:
-                return quint64(stats.tx_pkts());
+                return QString("%L1").arg(quint64(stats.tx_pkts()));
 
             case e_STAT_FRAME_SEND_RATE:
-                return quint64(stats.tx_pps());
+                return QString("%L1").arg(quint64(stats.tx_pps()));
 
             case e_STAT_FRAME_RECV_RATE:
-                return quint64(stats.rx_pps());
+                return QString("%L1").arg(quint64(stats.rx_pps()));
 
             case e_STAT_BYTES_RCVD:
-                return quint64(stats.rx_bytes());
+                return QString("%L1").arg(quint64(stats.rx_bytes()));
 
             case e_STAT_BYTES_SENT:
-                return quint64(stats.tx_bytes());
+                return QString("%L1").arg(quint64(stats.tx_bytes()));
 
             case e_STAT_BYTE_SEND_RATE:
-                return quint64(stats.tx_bps());
+                return QString("%L1").arg(quint64(stats.tx_bps()));
 
             case e_STAT_BYTE_RECV_RATE:
-                return quint64(stats.rx_bps());
+                return QString("%L1").arg(quint64(stats.rx_bps()));
 
 #if 0
             case e_STAT_FRAMES_RCVD_NIC:
@@ -170,10 +170,15 @@ QVariant PortStatsModel::data(const QModelIndex &index, int role) const
             case e_STAT_BYTES_SENT_NIC:
                 return stats.tx_bytes_nic();
 #endif
-            case e_STAT_RX_DROPS : return quint64(stats.rx_drops());
-            case e_STAT_RX_ERRORS: return quint64(stats.rx_errors());
-            case e_STAT_RX_FIFO_ERRORS: return quint64(stats.rx_fifo_errors());
-            case e_STAT_RX_FRAME_ERRORS: return quint64(stats.rx_frame_errors());
+
+            case e_STAT_RX_DROPS:
+                return QString("%L1").arg(quint64(stats.rx_drops()));
+            case e_STAT_RX_ERRORS:
+                return QString("%L1").arg(quint64(stats.rx_errors()));
+            case e_STAT_RX_FIFO_ERRORS:
+                return QString("%L1").arg(quint64(stats.rx_fifo_errors()));
+            case e_STAT_RX_FRAME_ERRORS:
+                return QString("%L1").arg(quint64(stats.rx_frame_errors()));
 
             default:
                 qWarning("%s: Unhandled stats id %d\n", __FUNCTION__,
