@@ -43,9 +43,12 @@ class StreamConfigDialog : public QDialog, public Ui::StreamConfigDialog
 {
     Q_OBJECT
 public:
-    StreamConfigDialog(Port &port, uint streamIndex, QWidget *parent = 0);
+    StreamConfigDialog(QList<Stream*> &streamList, const Port &port,
+            QWidget *parent = 0);
     ~StreamConfigDialog();
 
+    void setWindowTitle(const QString &title);
+
 private: 
 
     enum ButtonId
@@ -64,6 +67,8 @@ private:
         ProtoL4 = 4,
         ProtoL5 = 5,
         ProtoPayload = 6,
+        ProtoSign = 7,
+        ProtoTrailer = 8,
         ProtoMax
     };
 
@@ -72,7 +77,10 @@ private:
     QStringListModel *mpAvailableProtocolsModel;
     QStringListModel *mpSelectedProtocolsModel;
 
-    Port&            mPort;
+    QList<Stream*>  _userStreamList;
+    QList<Stream*>  _streamList;
+    const Port&     mPort;
+    QString         _windowTitle;
     uint            mCurrentStreamIndex;
 
     Stream                    *mpStream;
@@ -92,6 +100,7 @@ private:
     static int lastProtocolDataIndex;
 
     void setupUiExtra();
+    bool isCurrentStreamValid();
     void LoadCurrentStream();
     void StoreCurrentStream();
     void loadProtocolWidgets();
