@@ -18,193 +18,19 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
 
 #include "gmp.h"
-
-#include <QHeaderView>
-#include <qendian.h>
+#include <QStringList>
 
 QHash<int, int> GmpProtocol::frameFieldCountMap;
 
-GmpConfigForm::GmpConfigForm(QWidget *parent)
-    : QWidget(parent)
-{
-    setupUi(this);
-
-    auxData->setValidator(new QRegExpValidator(
-                QRegExp("[0-9A-Fa-f]*"), this));
-}
-
-GmpConfigForm::~GmpConfigForm()
-{
-}
-
-void GmpConfigForm::update()
-{
-    // save the current group Record by simulating a currentItemChanged()
-    on_groupList_currentItemChanged(groupList->currentItem(), 
-            groupList->currentItem());
-}
-
-void GmpConfigForm::on_groupMode_currentIndexChanged(int index)
-{
-    bool disabled = (index == 0);
-
-    groupCount->setDisabled(disabled);
-    groupPrefix->setDisabled(disabled);
-}
-
-void GmpConfigForm::on_addSource_clicked()
-{
-    QListWidgetItem *item=new QListWidgetItem(_defaultSourceIp);
-    item->setFlags(item->flags() | Qt::ItemIsEditable);
-    sourceList->insertItem(sourceList->currentRow(), item);
-
-    if (!overrideSourceCount->isChecked())
-        sourceCount->setText(QString().setNum(sourceList->count()));
-}
-
-void GmpConfigForm::on_deleteSource_clicked()
-{
-    delete sourceList->takeItem(sourceList->currentRow());
-
-    if (!overrideSourceCount->isChecked())
-        sourceCount->setText(QString().setNum(sourceList->count()));
-}
-
-void GmpConfigForm::on_addGroupRecord_clicked()
-{
-    OstProto::Gmp::GroupRecord defRec;
-    QVariantMap grpRec;
-    QListWidgetItem *item = new QListWidgetItem;
-
-    grpRec["groupRecordType"] = defRec.type();
-    grpRec["groupRecordAddress"] = _defaultGroupIp;
-    grpRec["overrideGroupRecordSourceCount"] =defRec.is_override_source_count();
-    grpRec["groupRecordSourceCount"] = defRec.source_count();
-    grpRec["groupRecordSourceList"] = QStringList();
-    grpRec["overrideAuxDataLength"] = defRec.is_override_aux_data_length();
-    grpRec["auxDataLength"] = defRec.aux_data_length();
-    grpRec["auxData"] = QByteArray().append(
-            QString().fromStdString(defRec.aux_data()));
-
-    item->setData(Qt::UserRole, grpRec);
-    item->setText(QString("%1: %2")
-            .arg(groupRecordType->itemText(grpRec["groupRecordType"].toInt()))
-            .arg(grpRec["groupRecordAddress"].toString()));
-
-    groupList->insertItem(groupList->currentRow(), item);
-
-    if (!overrideGroupRecordCount->isChecked())
-        groupRecordCount->setText(QString().setNum(groupList->count()));
-}
-
-void GmpConfigForm::on_deleteGroupRecord_clicked()
-{
-    delete groupList->takeItem(groupList->currentRow());
-
-    if (!overrideGroupRecordCount->isChecked())
-        groupRecordCount->setText(QString().setNum(groupList->count()));
-}
-
-void GmpConfigForm::on_groupList_currentItemChanged(QListWidgetItem *current,
-        QListWidgetItem *previous)
-{
-    QVariantMap rec;
-    QStringList strList;
-    
-    qDebug("in %s", __FUNCTION__);
-
-    // save previous record ...
-    if (previous == NULL)
-        goto _load_current_record;
-
-    rec["groupRecordType"] = groupRecordType->currentIndex();
-    rec["groupRecordAddress"] = groupRecordAddress->text();
-    strList.clear();
-    while (groupRecordSourceList->count())
-    {
-        QListWidgetItem *item = groupRecordSourceList->takeItem(0);
-        strList.append(item->text());
-        delete item;
-    }
-    rec["groupRecordSourceList"] = strList;
-    rec["overrideGroupRecordSourceCount"] = 
-                            overrideGroupRecordSourceCount->isChecked();
-    rec["groupRecordSourceCount"] = groupRecordSourceCount->text().toUInt();
-    rec["overrideAuxDataLength"] = overrideAuxDataLength->isChecked();
-    rec["auxDataLength"] = auxDataLength->text().toUInt();
-    rec["auxData"] = QByteArray().fromHex(QByteArray().append(auxData->text()));
-
-    previous->setData(Qt::UserRole, rec);
-    previous->setText(QString("%1: %2")
-            .arg(groupRecordType->itemText(rec["groupRecordType"].toInt()))
-            .arg(rec["groupRecordAddress"].toString()));
-
-_load_current_record:
-    // ... and load current record
-    if (current == NULL)
-        goto _exit;
-
-    rec = current->data(Qt::UserRole).toMap();
-
-    groupRecordType->setCurrentIndex(rec["groupRecordType"].toInt());
-    groupRecordAddress->setText(rec["groupRecordAddress"].toString());
-    strList = rec["groupRecordSourceList"].toStringList();
-    groupRecordSourceList->clear();
-    foreach (QString str, strList)
-    {
-        QListWidgetItem *item = new QListWidgetItem(str, groupRecordSourceList);
-        item->setFlags(item->flags() | Qt::ItemIsEditable);
-    }
-    overrideGroupRecordSourceCount->setChecked(
-                        rec["overrideGroupRecordSourceCount"].toBool());
-    groupRecordSourceCount->setText(QString().setNum(
-                        rec["groupRecordSourceCount"].toUInt()));
-    overrideAuxDataLength->setChecked(rec["overrideAuxDataLength"].toBool());
-    auxDataLength->setText(QString().setNum(rec["auxDataLength"].toUInt()));
-    auxData->setText(QString(rec["auxData"].toByteArray().toHex()));
-
-_exit:
-    groupRecord->setEnabled(current != NULL);
-    return;
-}
-
-void GmpConfigForm::on_addGroupRecordSource_clicked()
-{
-    QListWidgetItem *item=new QListWidgetItem(_defaultSourceIp);
-    item->setFlags(item->flags() | Qt::ItemIsEditable);
-    groupRecordSourceList->insertItem(groupRecordSourceList->currentRow(),item);
-
-    if (!overrideGroupRecordSourceCount->isChecked())
-        groupRecordSourceCount->setText(QString().setNum(
-                    groupRecordSourceList->count()));
-}
-
-void GmpConfigForm::on_deleteGroupRecordSource_clicked()
-{
-    delete groupRecordSourceList->takeItem(groupRecordSourceList->currentRow());
-
-    if (!overrideGroupRecordSourceCount->isChecked())
-        groupRecordSourceCount->setText(QString().setNum(
-                    groupRecordSourceList->count()));
-}
-
-void GmpConfigForm::on_auxData_textChanged(const QString &text)
-{
-    // auxDataLength is in units of words and each byte is 2 chars in text()
-    if (!overrideAuxDataLength->isChecked())
-        auxDataLength->setText(QString().setNum((text.size()+7)/8));
-}
-
 GmpProtocol::GmpProtocol(StreamBase *stream, AbstractProtocol *parent)
     : AbstractProtocol(stream, parent)
 {
-    /* The configWidget is created lazily */
-    configForm = NULL;
+    // field count may change based on msgType - so don't cache field offsets
+    _cacheFlags &= ~FieldFrameBitOffsetCache;
 }
 
 GmpProtocol::~GmpProtocol()
 {
-    delete configForm;
 }
 
 AbstractProtocol::ProtocolIdType GmpProtocol::protocolIdType() const
@@ -911,26 +737,12 @@ int GmpProtocol::protocolFrameSize(int streamIndex) const
     return AbstractProtocol::protocolFrameValue(streamIndex, true).size();
 }
 
-bool GmpProtocol::isProtocolFrameValueVariable() const
-{
-    // No fields vary for Ssm Query and Report
-    if (isSsmReport() || isSsmQuery())
-        return false;
-
-    // For all other msg types, check the group mode
-    if (fieldData(kGroupMode, FieldValue).toUInt() 
-            != uint(OstProto::Gmp::kFixed))
-        return true;
-
-    return false;
-}
-
 int GmpProtocol::protocolFrameVariableCount() const
 {
-    int count = 1;
+    int count = AbstractProtocol::protocolFrameVariableCount();
 
-    // No fields vary for Ssm Query and Report
-    if (isSsmReport() || isSsmQuery())
+    // No fields vary for Ssm Report
+    if (isSsmReport())
         return count;
 
     // For all other msg types, check the group mode
@@ -943,113 +755,3 @@ int GmpProtocol::protocolFrameVariableCount() const
 
     return count;
 }
-
-void GmpProtocol::loadConfigWidget()
-{
-    configWidget();
-
-    configForm->msgTypeCombo->setValue(fieldData(kType, FieldValue).toUInt());
-    // XXX: configForm->maxResponseTime set by subclass
-    configForm->overrideChecksum->setChecked(
-            fieldData(kIsOverrideChecksum, FieldValue).toBool());
-    configForm->checksum->setText(uintToHexStr(
-            fieldData(kChecksum, FieldValue).toUInt(), 2));
-
-    configForm->groupAddress->setText(
-            fieldData(kGroupAddress, FieldValue).toString());
-    configForm->groupMode->setCurrentIndex(
-            fieldData(kGroupMode, FieldValue).toUInt());
-    configForm->groupCount->setText(
-            fieldData(kGroupCount, FieldValue).toString());
-    configForm->groupPrefix->setText(
-            fieldData(kGroupPrefix, FieldValue).toString());
-
-    configForm->sFlag->setChecked(fieldData(kSFlag, FieldValue).toBool());
-    configForm->qrv->setText(fieldData(kQrv, FieldValue).toString());
-    configForm->qqi->setText(fieldData(kQqic, FieldValue).toString());
-
-    QStringList sl = fieldData(kSources, FieldValue).toStringList();
-    configForm->sourceList->clear();
-    foreach(QString src, sl)
-    {
-        QListWidgetItem *item = new QListWidgetItem(src);
-        item->setFlags(item->flags() | Qt::ItemIsEditable);
-        configForm->sourceList->addItem(item);
-    }
-
-    // NOTE: SourceCount should be loaded after sourceList
-    configForm->overrideSourceCount->setChecked(
-            fieldData(kIsOverrideSourceCount, FieldValue).toBool());
-    configForm->sourceCount->setText(
-            fieldData(kSourceCount, FieldValue).toString());
-
-    QVariantList list = fieldData(kGroupRecords, FieldValue).toList();
-    configForm->groupList->clear();
-    foreach (QVariant rec, list)
-    {
-        QVariantMap grpRec = rec.toMap();
-        QListWidgetItem *item = new QListWidgetItem;
-
-        item->setData(Qt::UserRole, grpRec);
-        item->setText(QString("%1: %2")
-                .arg(configForm->groupRecordType->itemText(
-                        grpRec["groupRecordType"].toInt()))
-                .arg(grpRec["groupRecordAddress"].toString()));
-        configForm->groupList->addItem(item);
-    }
-
-    // NOTE: recordCount should be loaded after recordList
-    configForm->overrideGroupRecordCount->setChecked(
-            fieldData(kIsOverrideGroupRecordCount, FieldValue).toBool());
-    configForm->groupRecordCount->setText(
-            fieldData(kGroupRecordCount, FieldValue).toString());
-
-}
-
-void GmpProtocol::storeConfigWidget()
-{
-    bool isOk;
-
-    configWidget();
-
-    configForm->update();
-
-    setFieldData(kType, configForm->msgTypeCombo->currentValue());
-    // XXX: configForm->maxResponseTime handled by subclass
-    setFieldData(kIsOverrideChecksum, 
-            configForm->overrideChecksum->isChecked());
-    setFieldData(kChecksum, 
-            configForm->checksum->text().toUInt(&isOk, BASE_HEX));
-
-    setFieldData(kGroupAddress, configForm->groupAddress->text());
-    setFieldData(kGroupMode, configForm->groupMode->currentIndex());
-    setFieldData(kGroupCount, configForm->groupCount->text());
-    setFieldData(kGroupPrefix, configForm->groupPrefix->text().remove('/'));
-
-    setFieldData(kSFlag, configForm->sFlag->isChecked());
-    setFieldData(kQrv, configForm->qrv->text());
-    setFieldData(kQqic, configForm->qqi->text());
-
-    QStringList list;
-    for (int i = 0; i < configForm->sourceList->count(); i++)
-        list.append(configForm->sourceList->item(i)->text());
-    setFieldData(kSources, list);
-
-    // sourceCount should be AFTER sources
-    setFieldData(kIsOverrideSourceCount, 
-            configForm->overrideSourceCount->isChecked());
-    setFieldData(kSourceCount, configForm->sourceCount->text());
-
-    QVariantList grpList;
-    for (int i = 0; i < configForm->groupList->count(); i++)
-    {
-        QVariant grp = configForm->groupList->item(i)->data(Qt::UserRole);
-        grpList.append(grp.toMap());
-    }
-    setFieldData(kGroupRecords, grpList);
-
-    // groupRecordCount should be AFTER groupRecords
-    setFieldData(kIsOverrideGroupRecordCount, 
-            configForm->overrideGroupRecordCount->isChecked());
-    setFieldData(kGroupRecordCount, configForm->groupRecordCount->text());
-}
