@@ -51,6 +51,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 #include "hexdumpconfig.h"
 #include "payloadconfig.h"
 #include "sampleconfig.h"
+#include "signconfig.h"
 #include "userscriptconfig.h"
 
 ProtocolWidgetFactory *OstProtocolWidgetFactory;
@@ -154,6 +155,9 @@ ProtocolWidgetFactory::ProtocolWidgetFactory()
     OstProtocolWidgetFactory->registerProtocolConfigWidget(
             OstProto::Protocol::kSampleFieldNumber, 
             (void*) SampleConfigForm::createInstance);
+    OstProtocolWidgetFactory->registerProtocolConfigWidget(
+            OstProto::Protocol::kSignFieldNumber,
+            (void*) SignConfigForm::createInstance);
     OstProtocolWidgetFactory->registerProtocolConfigWidget(
             OstProto::Protocol::kUserScriptFieldNumber, 
             (void*) UserScriptConfigForm::createInstance);
