@@ -46,14 +46,14 @@ PythonFileFormat::~PythonFileFormat()
     // Nothing to do
 }
 
-bool PythonFileFormat::openStreams(const QString /*fileName*/, 
+bool PythonFileFormat::open(const QString /*fileName*/,
         OstProto::StreamConfigList &/*streams*/, QString &/*error*/)
 {
     // NOT SUPPORTED!
     return false;
 }
 
-bool PythonFileFormat::saveStreams(const OstProto::StreamConfigList streams, 
+bool PythonFileFormat::save(const OstProto::StreamConfigList streams,
         const QString fileName, QString &error)
 {
     QFile file(fileName);
@@ -80,9 +80,8 @@ bool PythonFileFormat::saveStreams(const OstProto::StreamConfigList streams,
 
             refl->ListFields(protocol, &fields);
             for (uint k = 0; k < fields.size(); k++) {
-                // skip protocol_id field
-                if (fields.at(k)->number() == 
-                        OstProto::Protocol::kProtocolIdFieldNumber)
+                // skip non extension fields
+                if (!fields.at(k)->is_extension())
                     continue;
 
                 if (fields.at(k)->file()->name() !=
@@ -199,7 +198,8 @@ bool PythonFileFormat::saveStreams(const OstProto::StreamConfigList streams,
                         OstProto::Protocol::kProtocolIdFieldNumber)
                     continue;
                 QString pfx("    p.Extensions[X]");
-                pfx.replace("X", fields.at(k)->name().c_str());
+                pfx.replace(fields.at(k)->is_extension()? "X": "Extensions[X]",
+                        fields.at(k)->name().c_str());
                 writeFieldAssignment(out, pfx, protocol,
                         refl, fields.at(k));
             }
@@ -450,7 +450,7 @@ void PythonFileFormat::writeFieldAssignment(
             std::string val = fieldDesc->is_repeated() ?
                 refl->GetRepeatedStringReference(msg, fieldDesc, index, &val) :
                 refl->GetStringReference(msg, fieldDesc, &val);
-            QString escVal = escapeString(val.c_str());
+            QString escVal = escapeString(QString::fromStdString(val));
             if (val != fieldDesc->default_value_string())
                 out << fieldName << " = '" << escVal << "'\n";
             break;
