@@ -25,18 +25,19 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 
 static uchar msgBuf[4096];
 
-PbRpcChannel::PbRpcChannel(QHostAddress ip, quint16 port, 
+PbRpcChannel::PbRpcChannel(QString serverName, quint16 port,
                            const ::google::protobuf::Message &notifProto)
     : notifPrototype(notifProto)
 {
     isPending = false;
     pendingMethodId = -1;    // don't care as long as isPending is false
 
+    method = NULL;
     controller = NULL;
     done = NULL;
     response = NULL;
 
-    mServerAddress = ip;
+    mServerHost = serverName;
     mServerPort = port;
     mpSocket = new QTcpSocket(this);
 
@@ -75,12 +76,12 @@ void PbRpcChannel::establish()
 {
     qDebug("In %s", __FUNCTION__);
 
-    mpSocket->connectToHost(mServerAddress, mServerPort);
+    mpSocket->connectToHost(mServerHost, mServerPort);
 }
 
-void PbRpcChannel::establish(QHostAddress ip, quint16 port)
+void PbRpcChannel::establish(QString serverName, quint16 port)
 {
-    mServerAddress = ip;
+    mServerHost = serverName;
     mServerPort = port;
     establish();
 }
@@ -107,9 +108,10 @@ void PbRpcChannel::CallMethod(
     {
         RpcCall call;
         qDebug("RpcChannel: queueing rpc since method %d is pending;<----\n "
-                "queued method = %d\n"
+                "queued method = %d:%s\n"
                 "queued message = \n%s\n---->", 
-                pendingMethodId, method->index(), req->DebugString().c_str());
+                pendingMethodId, method->index(), method->name().c_str(),
+                req->DebugString().c_str());
 
         call.method = method;
         call.controller = controller;
@@ -128,7 +130,8 @@ void PbRpcChannel::CallMethod(
     if (!req->IsInitialized())
     {
         qWarning("RpcChannel: missing required fields in request <----");
-        qDebug("req = \n%s", req->DebugString().c_str());
+        qDebug("req = %s\n%s", method->input_type()->name().c_str(),
+                req->DebugString().c_str());
         qDebug("error = \n%s\n--->", req->InitializationErrorString().c_str());
 
         controller->SetFailed("Required fields missing");
@@ -137,6 +140,7 @@ void PbRpcChannel::CallMethod(
     }
 
     pendingMethodId = method->index();
+    this->method=method;
     this->controller=controller;
     this->done=done;
     this->response=response;
@@ -153,8 +157,10 @@ void PbRpcChannel::CallMethod(
         qDebug("client(%s) sending %d bytes <----", __FUNCTION__, 
                 PB_HDR_SIZE + len);
         BUFDUMP(msg, PB_HDR_SIZE);
-        qDebug("method = %d\n req = \n%s\n---->", 
-                method->index(), req->DebugString().c_str());
+        qDebug("method = %d:%s\n req = %s\n%s\n---->",
+                method->index(), method->name().c_str(),
+                method->input_type()->name().c_str(),
+                req->DebugString().c_str());
     }
 
     mpSocket->write(msg, PB_HDR_SIZE);
@@ -256,6 +262,11 @@ _top:
         }
 
         case PB_MSG_TYPE_RESPONSE:
+        {
+            static quint32 cumLen = 0;
+            static QByteArray buffer;
+            int l = 0;
+
             if (!isPending)
             {
                 qWarning("not waiting for response");
@@ -269,28 +280,59 @@ _top:
                 goto _error_exit;
             }
 
+            msgLen = 0;
+            while (cumLen < len)
+            {
+                if (inStream->Next((const void**)&msg, &msgLen) == false) {
+                    //qDebug("No more data or stream error");
+                    goto _exit;
+                }
+
+                l = qMin(msgLen, int(len - cumLen));
+                buffer.append(QByteArray((char*)msg, l));
+                cumLen += l;
+                //qDebug("%s: buffer rcvd %d/%d/%d", __PRETTY_FUNCTION__, l, cumLen, len);
+            }
+
+            if (l < msgLen) {
+                qDebug("read extra bytes after response; putting back");
+                inStream->BackUp(msgLen - l);
+            }
+
+#if 0 // not needed?
+            if (cumLen < len)
+                goto _exit;
+#endif
+
             if (len)
-                response->ParseFromBoundedZeroCopyStream(inStream, len);
+                response->ParseFromArray((const void*)buffer, len);
+
+            cumLen = 0;
+            buffer.resize(0);
 
             // Avoid printing stats
             if (method != 13)
             {
                 qDebug("client(%s): Received Msg <---- ", __FUNCTION__);
-                qDebug("method = %d\nresp = \n%s\n---->",
-                        method, response->DebugString().c_str());
+                qDebug("method = %d:%s\nresp = %s\n%s\n---->",
+                        method, this->method->name().c_str(),
+                        this->method->output_type()->name().c_str(),
+                        response->DebugString().c_str());
             }
 
             if (!response->IsInitialized())
             {
                 qWarning("RpcChannel: missing required fields in response <----");
-                qDebug("resp = \n%s", response->DebugString().c_str());
+                qDebug("resp = %s\n%s",
+                        this->method->output_type()->name().c_str(),
+                        response->DebugString().c_str());
                 qDebug("error = \n%s\n--->", 
                         response->InitializationErrorString().c_str());
 
                 controller->SetFailed("Required fields missing");
             }
             break;
-
+        }
         case PB_MSG_TYPE_ERROR:
         {
             static quint32 cumLen = 0;
@@ -387,6 +429,7 @@ _top:
     done->Run();
 
     pendingMethodId = -1;
+    this->method = NULL;
     controller = NULL;
     response = NULL;
     isPending = false;
@@ -396,9 +439,11 @@ _top:
     {
         RpcCall call = pendingCallList.takeFirst();
         qDebug("RpcChannel: executing queued method <----\n"
-               "method = %d\n"
-               "req = \n%s\n---->", 
-                call.method->index(), call.request->DebugString().c_str());
+               "method = %d:%s\n"
+               "req = %s\n%s\n---->",
+                call.method->index(), call.method->name().c_str(),
+                call.method->input_type()->name().c_str(),
+                call.request->DebugString().c_str());
         CallMethod(call.method, call.controller, call.request, call.response,
                 call.done);
     }
@@ -443,6 +488,7 @@ void PbRpcChannel::on_mpSocket_disconnected()
     qDebug("In %s", __FUNCTION__);
 
     pendingMethodId = -1;
+    method = NULL;
     controller = NULL;
     response = NULL;
     isPending = false;
