@@ -21,8 +21,10 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 
 #include "abstractport.h"
 
-#include "../common/streambase.h"
 #include "../common/abstractprotocol.h"
+#include "../common/streambase.h"
+#include "devicemanager.h"
+#include "packetbuffer.h"
 
 #include <QString>
 #include <QIODevice>
@@ -43,9 +45,12 @@ AbstractPort::AbstractPort(int id, const char *device)
     data_.set_is_exclusive_control(false);
 
     isSendQueueDirty_ = false;
+    rateAccuracy_ = kHighAccuracy;
     linkState_ = OstProto::LinkStateUnknown;
     minPacketSetSize_ = 1;
 
+    deviceManager_ = new DeviceManager(this);
+
     maxStatsValue_ = ULLONG_MAX; // assume 64-bit stats
     memset((void*) &stats_, 0, sizeof(stats_));
     resetStats();
@@ -53,12 +58,36 @@ AbstractPort::AbstractPort(int id, const char *device)
 
 AbstractPort::~AbstractPort()
 {
+    delete deviceManager_;
 }    
 
 void AbstractPort::init()
 {
 }    
 
+/*! Can we modify Port with these params? Should modify cause port dirty? */
+bool AbstractPort::canModify(const OstProto::Port &port, bool *dirty)
+{
+    bool allow = true;
+
+    *dirty = false;
+
+    if (port.has_transmit_mode()
+            && (port.transmit_mode() != data_.transmit_mode())) {
+        *dirty = true;
+        allow = !isTransmitOn();
+    }
+
+    if (port.has_is_tracking_stream_stats()
+            && (port.is_tracking_stream_stats()
+                    != data_.is_tracking_stream_stats())) {
+        *dirty = true;
+        allow = !isTransmitOn();
+    }
+
+    return allow;
+}
+
 bool AbstractPort::modify(const OstProto::Port &port)
 {
     bool ret = true;
@@ -76,6 +105,9 @@ bool AbstractPort::modify(const OstProto::Port &port)
     if (port.has_transmit_mode())
         data_.set_transmit_mode(port.transmit_mode());
 
+    if (port.has_is_tracking_stream_stats())
+        ret |= setTrackStreamStats(port.is_tracking_stream_stats());
+
     if (port.has_user_name()) {
         data_.set_user_name(port.user_name());
     }
@@ -83,6 +115,11 @@ bool AbstractPort::modify(const OstProto::Port &port)
     return ret;
 }    
 
+DeviceManager* AbstractPort::deviceManager()
+{
+    return deviceManager_;
+}
+
 StreamBase* AbstractPort::streamAtIndex(int index)
 {
     Q_ASSERT(index < streamList_.size());
@@ -144,6 +181,24 @@ void AbstractPort::addNote(QString note)
     data_.set_notes(notes.toStdString());
 }
 
+bool AbstractPort::setTrackStreamStats(bool enable)
+{
+    data_.set_is_tracking_stream_stats(enable);
+
+    return true;
+}
+
+AbstractPort::Accuracy AbstractPort::rateAccuracy()
+{
+    return rateAccuracy_;
+}
+
+bool AbstractPort::setRateAccuracy(Accuracy accuracy)
+{
+    rateAccuracy_ = accuracy;
+    return true;
+}
+
 void AbstractPort::updatePacketList()
 {
     switch(data_.transmit_mode())
@@ -197,10 +252,8 @@ void AbstractPort::updatePacketListSequential()
             case OstProto::StreamControl::e_su_bursts:
                 burstSize = streamList_[i]->burstSize();
                 x = AbstractProtocol::lcm(frameVariableCount, burstSize);
-                n = ulong(burstSize * streamList_[i]->burstRate() 
-                            * streamList_[i]->numBursts()) / x;
-                y = ulong(burstSize * streamList_[i]->burstRate() 
-                            * streamList_[i]->numBursts()) % x;
+                n = ulong(burstSize * streamList_[i]->numBursts()) / x;
+                y = ulong(burstSize * streamList_[i]->numBursts()) % x;
                 if (streamList_[i]->burstRate() > 0)
                 {
                     ibg = 1e9/double(streamList_[i]->burstRate());
@@ -357,11 +410,16 @@ void AbstractPort::updatePacketListInterleaved()
 
     qDebug("In %s", __FUNCTION__);
 
+    clearPacketList();
+    if (streamList_.size() == 0)
+    {
+        isSendQueueDirty_ = false;
+        return;
+    }
+
     // First sort the streams by ordinalValue
     qSort(streamList_.begin(), streamList_.end(), StreamBase::StreamLessThan);
 
-    clearPacketList();
-
     for (int i = 0; i < streamList_.size(); i++)
     {
         if (!streamList_[i]->isEnabled())
@@ -571,8 +629,8 @@ void AbstractPort::stats(PortStats *stats)
     stats->rxBytes = (stats_.rxBytes >= epochStats_.rxBytes) ?
                         stats_.rxBytes - epochStats_.rxBytes :
                         stats_.rxBytes + (maxStatsValue_ - epochStats_.rxBytes);
-    stats->rxPps = stats_.rxPps; 
-    stats->rxBps = stats_.rxBps; 
+    stats->rxPps = stats_.rxPps;
+    stats->rxBps = stats_.rxBps;
 
     stats->txPkts = (stats_.txPkts >= epochStats_.txPkts) ?
                         stats_.txPkts - epochStats_.txPkts :
@@ -580,8 +638,8 @@ void AbstractPort::stats(PortStats *stats)
     stats->txBytes = (stats_.txBytes >= epochStats_.txBytes) ?
                         stats_.txBytes - epochStats_.txBytes :
                         stats_.txBytes + (maxStatsValue_ - epochStats_.txBytes);
-    stats->txPps = stats_.txPps; 
-    stats->txBps = stats_.txBps; 
+    stats->txPps = stats_.txPps;
+    stats->txBps = stats_.txBps;
 
     stats->rxDrops = (stats_.rxDrops >= epochStats_.rxDrops) ?
                         stats_.rxDrops - epochStats_.rxDrops :
@@ -596,3 +654,130 @@ void AbstractPort::stats(PortStats *stats)
                         stats_.rxFrameErrors - epochStats_.rxFrameErrors :
                         stats_.rxFrameErrors + (maxStatsValue_ - epochStats_.rxFrameErrors);
 }
+
+void AbstractPort::streamStats(uint guid, OstProto::StreamStatsList *stats)
+{
+    if (streamStats_.contains(guid))
+    {
+        StreamStatsTuple sst = streamStats_.value(guid);
+        OstProto::StreamStats *s = stats->add_stream_stats();
+
+        s->mutable_stream_guid()->set_id(guid);
+        s->mutable_port_id()->set_id(id());
+
+        s->set_tx_pkts(sst.tx_pkts);
+        s->set_tx_bytes(sst.tx_bytes);
+        s->set_rx_pkts(sst.rx_pkts);
+        s->set_rx_bytes(sst.rx_bytes);
+    }
+}
+
+void AbstractPort::streamStatsAll(OstProto::StreamStatsList *stats)
+{
+    // FIXME: change input param to a non-OstProto type and/or have
+    // a getFirst/Next like API?
+    StreamStatsIterator i(streamStats_);
+    while (i.hasNext())
+    {
+        i.next();
+        StreamStatsTuple sst = i.value();
+        OstProto::StreamStats *s = stats->add_stream_stats();
+
+        s->mutable_stream_guid()->set_id(i.key());
+        s->mutable_port_id()->set_id(id());
+
+        s->set_tx_pkts(sst.tx_pkts);
+        s->set_tx_bytes(sst.tx_bytes);
+        s->set_rx_pkts(sst.rx_pkts);
+        s->set_rx_bytes(sst.rx_bytes);
+    }
+}
+
+void AbstractPort::resetStreamStats(uint guid)
+{
+    streamStats_.remove(guid);
+}
+
+void AbstractPort::resetStreamStatsAll()
+{
+    streamStats_.clear();
+}
+
+void AbstractPort::clearDeviceNeighbors()
+{
+    deviceManager_->clearDeviceNeighbors();
+    isSendQueueDirty_ = true;
+}
+
+void AbstractPort::resolveDeviceNeighbors()
+{
+    // For a user triggered 'Resolve Neighbors', the behaviour we want is
+    //   IP not in cache - send ARP/NDP request
+    //   IP present in cache, but unresolved - re-send ARP/NDP request
+    //   IP present in cache and resolved - don't sent ARP/NDP
+    //
+    // Device does not resend ARP/NDP requests if the IP address is
+    // already present in the cache, irrespective of whether it is
+    // resolved or not (this is done to avoid sending duplicate requests).
+    //
+    // So, to get the behaviour we want, let's clear all unresolved neighbors
+    // before calling resolve
+    deviceManager_->clearDeviceNeighbors(Device::kUnresolvedNeighbors);
+
+    // Resolve gateway for each device first ...
+    deviceManager_->resolveDeviceGateways();
+
+    // ... then resolve neighbor for each unique frame of each stream
+    // NOTE:
+    // 1. All the frames may have the same destination ip,but may have
+    // different source ip so may belong to a different emulated device;
+    // so we cannot optimize and send only one ARP
+    // 2. For a unidirectional stream, at egress, this will create ARP
+    // entries on the DUT for each of the source addresses
+    //
+    // TODO(optimization): Identify if stream does not vary in srcIp or dstIp
+    // - in which case resolve for only one frame of the stream
+    for (int i = 0; i < streamList_.size(); i++)
+    {
+        const StreamBase *stream = streamList_.at(i);
+        int frameCount = stream->frameVariableCount();
+
+        for (int j = 0; j < frameCount; j++) {
+            // we need the packet contents only uptil the L3 header
+            int pktLen = stream->frameValue(pktBuf_, kMaxL3PktSize, j);
+            if (pktLen) {
+                PacketBuffer pktBuf(pktBuf_, pktLen);
+                deviceManager_->resolveDeviceNeighbor(&pktBuf);
+            }
+        }
+    }
+    isSendQueueDirty_ = true;
+}
+
+quint64 AbstractPort::deviceMacAddress(int streamId, int frameIndex)
+{
+    // we need the packet contents only uptil the L3 header
+    StreamBase *s = stream(streamId);
+    int pktLen = s->frameValue(pktBuf_, kMaxL3PktSize, frameIndex);
+
+    if (pktLen) {
+        PacketBuffer pktBuf(pktBuf_, pktLen);
+        return deviceManager_->deviceMacAddress(&pktBuf);
+    }
+
+    return 0;
+}
+
+quint64 AbstractPort::neighborMacAddress(int streamId, int frameIndex)
+{
+    // we need the packet contents only uptil the L3 header
+    StreamBase *s = stream(streamId);
+    int pktLen = s->frameValue(pktBuf_, kMaxL3PktSize, frameIndex);
+
+    if (pktLen) {
+        PacketBuffer pktBuf(pktBuf_, pktLen);
+        return deviceManager_->neighborMacAddress(&pktBuf);
+    }
+
+    return 0;
+}
