@@ -19,8 +19,10 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 
 #include "drone.h"
 
-#include "rpcserver.h"
 #include "myservice.h"
+#include "rpcserver.h"
+#include "settings.h"
+#include "../common/updater.h"
 
 #include <QMetaType>
 
@@ -31,8 +33,15 @@ extern const char* revision;
 Drone::Drone(QObject *parent)
      : QObject(parent)
 {
+    Updater *updater = new Updater();
+
     rpcServer = new RpcServer();
     service = new MyService();
+
+    connect(updater, SIGNAL(newVersionAvailable(QString)),
+            this, SLOT(onNewVersion(QString)));
+    updater->checkForNewVersion();
+
 }
 
 Drone::~Drone()
@@ -43,11 +52,21 @@ Drone::~Drone()
 
 bool Drone::init()
 {
+    QString addr = appSettings->value(kRpcServerAddress).toString();
+    QHostAddress address = addr.isEmpty() ?
+        QHostAddress::Any : QHostAddress(addr);
+
     Q_ASSERT(rpcServer);
 
     qRegisterMetaType<SharedProtobufMessage>("SharedProtobufMessage");
 
-    if (!rpcServer->registerService(service, myport ? myport : 7878))
+    if (address.isNull()) {
+        qWarning("Invalid RpcServer Address <%s> specified. Using 'Any'",
+                qPrintable(addr));
+        address = QHostAddress::Any;
+    }
+
+    if (!rpcServer->registerService(service, address, myport ? myport : 7878))
     {
         //qCritical(qPrintable(rpcServer->errorString()));
         return false;
@@ -58,3 +77,16 @@ bool Drone::init()
 
     return true;
 }
+
+MyService* Drone::rpcService()
+{
+    return service;
+}
+
+void Drone::onNewVersion(QString newVersion)
+{
+    qWarning("%s", qPrintable(QString("New Ostinato version %1 available. "
+                "Visit http://ostinato.org to download").arg(newVersion)));
+}
+
+
